﻿Imports System.Runtime.CompilerServices
Imports System.Data.Common
Imports System.Reflection
Imports System.ComponentModel
Imports System.Text.RegularExpressions
Imports System.IO
Imports System.Text

Namespace System.Data

   Public Module DataCommonX

#Region "ConnectionOpening"
      Private _ConnectionWatchers As New ComponentProperty(Of DbConnection, ConditionalActionFrame)(AddressOf CreateActionFrame)

      Private Function CreateActionFrame(conn As DbConnection) As ConditionalActionFrame
         Return New ConditionalActionFrame(Function() conn.State = ConnectionState.Closed, AddressOf conn.Open, AddressOf conn.Close)
      End Function

      <DebuggerStepThrough(), Extension()> _
      Public Function EnterOpen(conn As DbConnection) As IDisposable
         Return _ConnectionWatchers(conn).Enter()
      End Function

#End Region 'ConnectionOpening

#Region "Data.Common-Objekte klonen"

      <Extension()> _
      Public Function Clone(cmd As DbCommand) As DbCommand
         Return DirectCast(DirectCast(cmd, ICloneable).Clone(), DbCommand)
      End Function
      <Extension()> _
      Public Function Clone(param As DbParameter) As DbParameter
         Return DirectCast(DirectCast(param, ICloneable).Clone(), DbParameter)
      End Function
#End Region 'Data.Common-Objekte klonen

      <Extension()> _
      Public Sub Save(dts As DataSet)
         dts.WriteXml(GetDefaultName(dts))
      End Sub

      <Extension()> _
      Public Sub FillWpf(dts As DataSet)
         dts.Clear()
         dts.ReadXml(GetDefaultName(dts))
      End Sub

      Private Function GetDefaultName(dts As DataSet) As String
#If DEBUG Then
         Return New DirectoryInfo("..\..\").Combine(dts.GetType.Name & ".xml")
#Else
         Dim sDir = Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData)
         Return New DirectoryInfo(sDir).Combine(dts.GetType.Name & ".xml")
#End If
      End Function

      Public Enum CommandIdentType : Text = 1 : ParamTypes : ParamValues = 4 : All = 7 : End Enum
      ''' <summary> for Debugging: shows CommandText and Params </summary>
      <Extension()> _
      Public Function Identify(ByVal cmd As DbCommand, Optional identType As CommandIdentType = CommandIdentType.All) As String
         If identType = CommandIdentType.Text Then Return cmd.CommandText
         Dim matchCounter = 0
         Dim replacer As MatchEvaluator = _
            Function(mt)
               Dim param = If(mt.Value = "?", cmd.Parameters(matchCounter), cmd.Parameters(mt.Value))
               Dim sb As New StringBuilder()
               Dim netType = If(param.Value Is Nothing, "#", param.Value.GetType.Name)
               If identType.HasFlag(CommandIdentType.ParamTypes) Then
                  sb.Append("/".Between(param.DbType, netType))
                  If identType.HasFlag(CommandIdentType.ParamValues) Then sb.Append(": ")
               End If
               If identType.HasFlag(CommandIdentType.ParamValues) Then sb.Append(If(param.Value Is Nothing, "#", param.Value.ToString))
               matchCounter += 1
               Return mt.Value & sb.ToString.Between("{}")
            End Function
         Return Regex.Replace(cmd.CommandText, "@\w+|\?", replacer)
      End Function

      ''' <summary> befüllt die DbParameter eines DbCommands mit Werten </summary>
      <Extension()> _
      Public Function FillParams(ByVal cmdWithParams As DbCommand, ByVal ParamArray values As Object()) As DbCommand
         If cmdWithParams.Parameters.Count <> values.Length Then Throw New ArgumentException( _
         "Anzahl der Argumente passt nicht zur Anzahl der Parameter")
         For i = 0 To values.Length - 1
            cmdWithParams.Parameters(i).Value = values(i)
         Next
         Return cmdWithParams
      End Function
      ''' <summary> erzeugt ein DBCommand mit DBParametern. '?' im commandText ist Platzhalter für einen DBParameter </summary>
      <Extension()> _
      Public Function CreateCommandWithParams(ByVal con As DbConnection, ByVal commandText As String) As DbCommand
         Dim splits = commandText.Split("?"c)
         Dim cmd = con.CreateCommand
         For i = 0 To splits.Length - 2
            Dim param = cmd.CreateParameter
            param.ParameterName = "@a" & i
            cmd.Parameters.Add(param)
         Next
         If TypeOf con Is OleDb.OleDbConnection Then
            cmd.CommandText = commandText
         Else
            For i = 0 To splits.Length - 2
               splits(i) = splits(i) & cmd.Parameters(i).ParameterName
            Next
            cmd.CommandText = String.Concat(splits)
         End If
         Return cmd
      End Function
      ''' <summary> erzeugt ein DBCommand inklusive bereits mit Werten beschickter DBParameter. '?' im commandText ist Platzhalter für einen DBParameter </summary>
      <Extension()> _
      Public Function CreateCompleteCommand(ByVal con As DbConnection, ByVal commandText As String, ByVal ParamArray values As Object()) As DbCommand
         Return con.CreateCommandWithParams(commandText).FillParams(values)
      End Function

      ''' <summary> erzeugt ein DBCommand inklusive bereits mit Werten beschickter DBParameter. '?' im commandText ist Platzhalter für einen DBParameter </summary>
      <Extension()> _
      Public Sub WriteToDb(con As DbConnection, tb As DataTable, Optional clearFirst As Boolean = False)
         Dim ns = con.GetType.Namespace
         Dim factory = System.Data.Common.DbProviderFactories.GetFactory(ns)
         Using adp = factory.CreateDataAdapter, x = con.EnterOpen, builder = factory.CreateCommandBuilder, builder2 = factory.CreateCommandBuilder, tr = con.BeginTransaction, cmd = con.CreateCommand
            cmd.Transaction = tr
            adp.SelectCommand = cmd

            builder.DataAdapter = adp
            With builder
               .SetAllValues = True
               If .QuotePrefix.Length = 0 Then
                  Dim s = .QuoteIdentifier("")
                  .QuotePrefix = s(0)
                  .QuoteSuffix = s(1)
               End If
            End With
            If clearFirst Then
               cmd.CommandText = "Delete * from  " & builder.QuoteIdentifier(tb.TableName)
               cmd.ExecuteNonQuery()
            End If
            cmd.CommandText = "Select * from " & builder.QuoteIdentifier(tb.TableName)
            Dim n = adp.Update(tb)
            tr.Commit()
         End Using
      End Sub

   End Module

End Namespace
