﻿Imports System.ComponentModel
Imports System.Windows.Media
Imports System.Collections.ObjectModel
Imports System.Text.RegularExpressions
Imports System.Globalization

Partial Class DragablePath

   Private Class PathDataConverter

#Region "ClassHeader"
#If False Then
eigene Klasse, um zu kapseln, wie eine PathFigureCollection in smarte Markup-Syntax gebracht wird, also mit LineFeeds, Einrückungen und v.a. relativen Punkt-Koordinaten.
Zugrunde liegt das Markup, wie es der PathFigureCollectionTypeConverter produziert: ein einzeiliger String mit absoluten Koordinaten.
Umrechnen in relative ist nun aufwändig, denn es muss tw zw. LinienPunkt und SteuerPunkt unterschieden werden.
ZB bei PolyLine ist jeder Point ein LinienPunkt, und die Relativierung zieht von jedem Point die Absolut-Position seines Vorgängers ab - (Also Absolut-Position zuvor zu merken - für den nächsten).
Bei PolyBezier aber ist nur jeder 3. Punkt LinienPunkt - dazwischen liegen 2 SteuerPunkte. Von den SteuerPunkten muss auch die Absolut-Position des vorherigen LinienPunkts abgezogen werden, aber die SteuerPunkt-Absolut-Position nicht merken, sondern nur Linienpunkte sind Referenz-Punkte.
Also führe ich _LinePointIntervals ein, welches den SegmentTypen LQC verschiedene Intervalle zuordnet, und der Algo unterscheidet SteuerPunkt und LinienPunkt, und merkt nur LinienPunkte als Absolut-Position für die folgenden.
Arcsegment-'A' ist nochmal anders, denn da bedeuten die Komma-getrennten Zahlen alles mögliche andere als Point - nur das letzte Zahlen-Pärchen ist als LinienPunkt zu merken und zu relativieren.
Poly-ArcSegmente treten zum Glück garnet auf. Also man kann sie zwar formulieren, aber der Converter erzeugt daraus keine besonderen Poly-Segmente, sondern mehrere Arcs.
Um bei den aus 7 Zahlen bestehenden Parametern von A den Point am Ende richtig zu identifizieren durchläuft rgxPoint die Zeile rückwärts - daher muss der Algo auch rückwärts - und das macht sich zur Identifikation der LinePoints mit Modulo sogar besonders gut.
#End If
#End Region 'ClassHeader

      Public Shared Function GetPathFigureCollection(_PathData As String) As PathFigureCollection
         Return DirectCast(_Converter.ConvertFromInvariantString(_PathData), PathFigureCollection)
      End Function

      Public Shared Sub GetDataStrings(Figures As PathFigureCollection, ByRef rawString As String, ByRef smartString As String)
         Dim sRaw = _Converter.ConvertToInvariantString(Figures)
         rawString = rgxGetCommandLine.Replace(sRaw, " $0") 'zumindest Spaces vor die Commands einfügen
         smartString = GetSmartString(sRaw)
      End Sub

      Private Shared _Converter As TypeConverter = TypeDescriptor.GetConverter(GetType(PathFigureCollection))
      Private Shared _LinePointIntervals As String = " LQC"  ' Zuordnung (per .IndexOf()) von L Q C zu den linePointIntervals 1 2 3
      Private Shared rgxGetCommandLine As New Regex("[zA-Z][^zA-Z]*")
      Private Shared rgxPoint As New Regex("[-\.\d]+,[-\.\d]+", RegexOptions.RightToLeft) 'matcht 2 komma-getrennte doubles - rückwärts!

      Private Shared Function GetSmartString(sRaw As String) As String
         'although I calculate with Vectors, I name them "pt"
         Dim output As New List(Of String)
         For Each mtLine As Match In rgxGetCommandLine.Matches(sRaw)
            Dim line = mtLine.Value
            Dim ptAbs As Vector
            Dim commandAbs = line(0)
            Select Case commandAbs
               Case "z"c
                  output.Add("z")
               Case "M"c
                  Dim s = line.Substring(1)
                  ptAbs = Vector.Parse(s)
                  output.Add("M " & s)
               Case "A"c
                  Dim mt = rgxPoint.Match(line)
                  Dim pt = Vector.Parse(mt.Value)
                  output.Add("   a " & line.Substring(1, mt.Index - 1))
                  output.Add(" " & (pt - ptAbs).ToString(CultureInfo.InvariantCulture)) 'add transformed point
                  ptAbs = pt
               Case Else
                  Dim linePointInterval = _LinePointIntervals.IndexOf(commandAbs)
                  output.Add("   " & Char.ToLower(commandAbs)) '.Tolower ist das Command für relative Koordinaten
                  Dim mts = rgxPoint.Matches(line)
                  For i = mts.Count - 1 To 0 Step -1
                     Dim pt = Vector.Parse(mts(i).Value)
                     output.Add(" " & (pt - ptAbs).ToString(CultureInfo.InvariantCulture)) 'add transformed point
                     If i Mod linePointInterval = 0 Then ptAbs = pt 'on interval pt is the new reference-point (pt is segment-target)
                  Next
            End Select
            output.Add(Lf)
         Next
         Return String.Concat(output)
      End Function

   End Class

   Public Class PathFigureWrapper
      Public Iterator Function GetDragPoints() As IEnumerable(Of DragPoint)
         Yield _StartDragPoint
         For Each pt In _Segments.SelectMany(Function(seg) seg.Points)
            Yield pt
         Next
      End Function
      Private WithEvents _StartDragPoint As DragPoint
      Private _Segments As New List(Of SegmentWrapper)
      Private _Figure As PathFigure
      Public ReadOnly Property Figure() As PathFigure
         Get
            Return _Figure
         End Get
      End Property
      Public Sub New(Figure As PathFigure)
         _Figure = Figure
         _StartDragPoint = New DragPoint(_Figure.StartPoint)
         For Each segm In _Figure.Segments
            _Segments.Add(SegmentWrapper.FromPathSegment(segm))
         Next
      End Sub

      Private Sub _StartDragPoint_PropertyChanged(sender As Object, e As PropertyChangedEventArgs) Handles _StartDragPoint.PropertyChanged
         _Figure.StartPoint = _StartDragPoint.Point
      End Sub

   End Class
End Class

Public Class DragPoint : Inherits NotifyPropertyChanged

   Private _Point As Point
   Public Property Point() As Point
      Get
         Return _Point
      End Get
      Set(ByVal value As Point)
         ChangePropIfDifferent(value, _Point)
      End Set
   End Property

   Public Sub New(pt As Point)
      _Point = pt
   End Sub

End Class