﻿Imports System.Windows.Media
Imports System.ComponentModel
Partial Class DragablePath
   Public MustInherit Class SegmentWrapper
      Public Points As New BindingList(Of DragPoint)
      Private PointCollection As PointCollection
      Protected Sub SetPointCollection(points As PointCollection)
         PointCollection = points
         points.ForEach(Sub(pt) Me.Points.Add(New DragPoint(pt)))
         AddHandler Me.Points.ListChanged, AddressOf _Points_ListChanged
      End Sub
      Private Sub _Points_ListChanged(sender As Object, e As ListChangedEventArgs)
         If e.ListChangedType <> ListChangedType.ItemChanged Then Return
         PointCollection(e.NewIndex) = Points(e.NewIndex).Point
      End Sub
      Protected Sub AddSegmentPoint(pt As Point, action As PropertyChangedEventHandler)
         Dim dp = New DragPoint(pt)
         Points.Add(dp)
         AddHandler dp.PropertyChanged, action
      End Sub
      Public Shared Function FromPathSegment(segm As PathSegment) As SegmentWrapper
         With TryCast(segm, LineSegment)
            If .NotNull Then Return New LineWrapper(.Self)
         End With
         With TryCast(segm, PolyLineSegment)
            If .NotNull Then Return New PolyLineWrapper(.Self)
         End With
         With TryCast(segm, BezierSegment)
            If .NotNull Then Return New BezierWrapper(.Self)
         End With
         With TryCast(segm, QuadraticBezierSegment)
            If .NotNull Then Return New QuadraticBezierWrapper(.Self)
         End With
         With TryCast(segm, PolyQuadraticBezierSegment)
            If .NotNull Then Return New PolyQuadraticBezierWrapper(.Self)
         End With
         With TryCast(segm, PolyBezierSegment)
            If .NotNull Then Return New PolyBezierWrapper(.Self)
         End With
         With TryCast(segm, ArcSegment)
            If .NotNull Then Return New ArcWrapper(.Self)
         End With
         Throw New NotSupportedException
      End Function
   End Class

   Public Class PolyLineWrapper : Inherits SegmentWrapper
      Public Sub New(Segment As PolyLineSegment)
         SetPointCollection(Segment.Points) : Return
      End Sub
   End Class
   Public Class PolyBezierWrapper : Inherits SegmentWrapper
      Public Sub New(Segment As PolyBezierSegment)
         SetPointCollection(Segment.Points) : Return
      End Sub
   End Class
   Public Class PolyQuadraticBezierWrapper : Inherits SegmentWrapper
      Public Sub New(Segment As PolyQuadraticBezierSegment)
         SetPointCollection(Segment.Points) : Return
      End Sub
   End Class

   Public Class ArcWrapper : Inherits SegmentWrapper
      Private _Segment As ArcSegment
      Public Sub New(Segment As ArcSegment)
         _Segment = Segment
         AddSegmentPoint(_Segment.Point, Sub(s, e) _Segment.Point = DirectCast(s, DragPoint).Point)
      End Sub
   End Class
   Public Class LineWrapper : Inherits SegmentWrapper
      Private _Segment As LineSegment
      Public Sub New(Segment As LineSegment)
         _Segment = Segment
         AddSegmentPoint(_Segment.Point, Sub(s, e) _Segment.Point = DirectCast(s, DragPoint).Point)
      End Sub
   End Class
   Public Class QuadraticBezierWrapper : Inherits SegmentWrapper
      Private _Segment As QuadraticBezierSegment
      Public Sub New(Segment As QuadraticBezierSegment)
         _Segment = Segment
         AddSegmentPoint(_Segment.Point1, Sub(s, e) _Segment.Point1 = DirectCast(s, DragPoint).Point)
         AddSegmentPoint(_Segment.Point2, Sub(s, e) _Segment.Point2 = DirectCast(s, DragPoint).Point)
      End Sub
   End Class
   Public Class BezierWrapper : Inherits SegmentWrapper
      Private _Segment As BezierSegment
      Public Sub New(Segment As BezierSegment)
         _Segment = Segment
         AddSegmentPoint(_Segment.Point1, Sub(s, e) _Segment.Point1 = DirectCast(s, DragPoint).Point)
         AddSegmentPoint(_Segment.Point2, Sub(s, e) _Segment.Point2 = DirectCast(s, DragPoint).Point)
         AddSegmentPoint(_Segment.Point3, Sub(s, e) _Segment.Point3 = DirectCast(s, DragPoint).Point)
      End Sub
   End Class

End Class