﻿#Region "FileHeader"
#If False Then
Es wird kunstvoll ein SmartFormatted-String mit einem ArcSegment generiert, an das ein Path binden kann. Das ArcSegment zeichnet von nix bis Vollkreis, je nach Value-Wert der RoundScala.
Generiert wird ein fixer GrößenBereich - der Arc ist Segment des darin eingeschriebenen Kreises. Die Scalierung kann man dann im Xaml machen, nämlich mit Path.Stretch = Fill
Die Rechnerei basiert auf 3 Standard-Vektoren: vMin, vMax, vStart, die das Einheits-Längenmass - 50 - verschieden verwenden.
vMin, vMax spannen den Bereich auf (obenLinks, untenRechts), und vStart ist obenMitte.
Der Arc beschreibt dann die Kreislinie von vStart zum Zielpunkt, welcher nix anderes ist als der um den Nullpunkt rotierte vStart.
Die Matrix-Transformation rotiert vStart um Value * 360° - also bei Value = 1 ist Vollkreis.

ZB Value = 0.25 generiert dieses (Viertel-Kreis-) Data: 
M-50,-50 M50,50 M0,-50 A50,50 0 0 1 50,0
Bedeutung:
M-50,-50: oben links (vMin)
M50,50: unten rechts (vMax)
M0,-50: oben mitte - (vStart)

dann folgt die Arc-Definition:
A50,50 0 0 1 50,0    - hierbei bedeuten:
50,50: x-/y-Radius der Ellipse (identisch mit vMax)
0: keine Rotation
0: largeArc=False, es wird also der kürzere Ellipsen-Bogen zum Zielpunkt gezeichnet (sonst entstünde ein 3/4-Kreis)
1: Clockwise=true
50,0: Zielpunkt auf der Kreislinie rechts mittig, alias 90° Randpunkt, alias 1/4 (d.h. 0.25) des Vollkreises.

Nochmal allg. Arc-Syntax: 
A{x-radius,y-radius} {rotation} {largeArc} {clockwise} {targetX, targetY}
Der Startpunkt ist nicht Syntax-Bestandteil: Path-Segmente starten immer auf der vorherigen Position, die daher zuvor mit M0,-50 bereitzustellen ist.

Also an unserem Data ist nur largeArc und Zielpunkt variabel, daher erzeuge ich einen shared Format-String, mit Platzhalter für diese beiden:
M-50,-50 M50,50 M0,-50 A50,50 0 {0} 1 {1}

#End If '-- Options, Imports
Imports System.ComponentModel
Imports System.Windows.Media
Imports System.Globalization
#End Region 'FileHeader


Public Class RoundScala : Inherits NotifyPropertyChanged

   Public Sub New()
      Value = 0.25
   End Sub

   Private Shared ReadOnly _Format As String
   Private Shared ReadOnly _vStart As Vector

   Shared Sub New()
      Dim vMax As New Vector(50, 50)
      Dim vMin As Vector = vMax * -1
      _vStart = New Vector(0, vMin.Y)
      _Format = String.Format(CultureInfo.InvariantCulture, "M{0} M{1} M{2} A{1} 0 {{0}} 1 {{1}}", vMin, vMax, _vStart)
   End Sub

   Private _Value As Double
   Public Property Value() As Double
      Get
         Return _Value
      End Get
      Set(ByVal value As Double)
         If Not ChangePropIfDifferent(value, _Value) Then Return
         Dim angle = _Value * 359.999 ' micro-gap, because on exact 360° the path would vanish.
         Dim isLargeArc = Convert.ToInt32(angle >= 180) ' ab 180° muss die längere Verbindung zum Zielpunkt gezeichnet werden
         Dim _Matrix As New Matrix
         _Matrix.Rotate(angle)
         Data = String.Format(CultureInfo.InvariantCulture, _Format, isLargeArc, _Matrix.Transform(_vStart))
      End Set
   End Property

   Private _Data As String
   Public Property Data As String
      Get
         Return _Data
      End Get
      Set(ByVal value As String)
         ChangePropIfDifferent(value, _Data)
      End Set
   End Property

End Class
