﻿Imports System.ComponentModel

Namespace System.Windows.Input
   Public Class CommandTree
      Inherits CustomTypeDescriptor
      Implements IEnumerable(Of TreeCommand)

      Public ReadOnly Root As TreeCommand

      Private Enum CharIndexMapping
         _0 = 48
         _10 = 87
         _MaxIndex = 65438
      End Enum

      Public Sub New(ParamArray executeMethods As Action())
         Root = TreeCommand.Create("Nop", AddressOf Nop)
         Root.ID = ""
         Root.AddRangeFlow(executeMethods)
      End Sub

      ''' <summary> associate ints to chars, assuming a "charset" of: 0123456789abcdefghi... upto char.MaxValue. The resulting MaxIndex is 65438 </summary>
      Private Shared Function GetIdCharFromNodeIndex(index As Integer) As Char
         If index < 0 Then
            Throw New IndexOutOfRangeException("index must be >= 0")
         ElseIf index < 10 Then
            Return Convert.ToChar(CharIndexMapping._0 + index)
         ElseIf index <= CInt(CharIndexMapping._MaxIndex) Then
            Return Convert.ToChar(CharIndexMapping._10 + index)
         Else
            Throw New IndexOutOfRangeException("index must be <= " & CInt(CharIndexMapping._MaxIndex))
         End If
      End Function

      ''' <summary> set a unique ID to each TreeCommand, return all Commands as flat IEnumerable. The ID is hierarchical and human readable. Eg "021" addresses the first node, and from that the third child, and from that the second child </summary>
      Public Function SetupIDs() As IEnumerable(Of TreeCommand)
         'breath-first-traversal with assigning new IDs
         Dim lst = New List(Of TreeCommand)()
         Dim addRange As Action(Of TreeCommand) = Sub(parent)
                                                     Dim childs = parent.Children
                                                     Dim id = parent.ID
                                                     For i = 0 To childs.Count - 1
                                                        childs(i).ID = id + GetIdCharFromNodeIndex(i)
                                                     Next
                                                     lst.AddRange(childs)
                                                  End Sub
         addRange(Root)
         For i = 0 To lst.Count - 1
            addRange(lst(i))
         Next
         Return lst.Distinct()
      End Function

      ''' <summary> for internal use, to support Databinding </summary>
      <EditorBrowsable(EditorBrowsableState.Never)> _
      Public Overrides Function GetProperties() As PropertyDescriptorCollection
         'unconventional, but efficient: the PropertyDescriptor is a Member of the Property-Datatype
         Return New PropertyDescriptorCollection(SetupIDs().Select(Function(cmd) cmd.Descriptor).ToArray())
      End Function
      ''' <summary> for internal use, to support Databinding </summary>
      <EditorBrowsable(EditorBrowsableState.Never)> _
      Public Overrides Function GetProperties(attributes As Attribute()) As PropertyDescriptorCollection
         Return GetProperties()
      End Function

      Private Function IEnumerable_GetEnumerator() As IEnumerator(Of TreeCommand) Implements IEnumerable(Of TreeCommand).GetEnumerator
         Return Root.Children.GetEnumerator()
      End Function

      Private Function Collections_IEnumerable_GetEnumerator() As Collections.IEnumerator Implements Collections.IEnumerable.GetEnumerator
         Return Root.Children.GetEnumerator()
      End Function
   End Class
End Namespace
