﻿#Region "FileHeader"
#If False Then
Viewmodel, an das man Menu/ContextMenu binden kann. Man verwende aber kein HierarchicalDataTemplate, sondern binde die MenuItems - inklusive ItemsSource im ItemsContainerStyle.
Man kann auch ItemsControls mit Buttons drinne anbinden, aber wenn manche MenuNodes eine ImageSource haben, und andere nicht, so muss man dem Button einen triggernden Style geben, der dem Rechnung trägt.
Denn Button hat keine Image-Property wie MenuItem, sondern man muss einen Content gestalten, mit Image, gebunden an hier ImageSource.
Zur Nutzung beerbe man MenuTree, und definiere einen Haufen Shared Strings. Diese werden per Reflection ausgelesen und werden zur ID, aber auch zur Beschriftug, falls kein String-Wert angegeben ist.
Die Command-Actions werden in Events umgeleitet, sodass man im Viewmodel einen zentralen Handler aller Button-Klickse führen kann, die man anhand der ID unterscheiden kann, und die auch Parameter durchreichen können.
Anwendungsbeispiel:
   Public Class Menu : Inherits MenuTree
      Public Shared _Load, _Save, _File, _Action, _Push, Pu_ll, _Merge As String
      Public Shared ExitNoSave As String = "E_xit Without Saving Changes", SyncOptions As String = "_Sync-Options"
      Public Sub New()
         SubMenu(_File, _Load, _Save, ExitNoSave)(_Action, _Push, Pu_ll, _Merge)
         dim o=SetImage(ExitNoSave, "NoSaveIcon.ico")(_Save, "SaveIcon.ico")
      End Sub
   End Class
   Public WithEvents MainMenu As New Menu

   Private Sub MainMenu_Execute(sender As Object, e As ExecuteEventargs) Handles MainMenu.Execute
      Select Case e.ID
         Case Menu.ExitNoSave : End
         Case Menu._Load : Reload()
         Case Menu._Save : Save()
         Case Menu._Push : LaunchSync(SyncMode.Push)
         Case Menu.Pu_ll : LaunchSync(SyncMode.Pull)
         Case Menu._Merge : LaunchSync(SyncMode.Merge)
         Case Menu.SyncOptions : MessageBox.Show(My.Settings.SyncOptions, "Synchronisize-Options")
      End Select
   End Sub
#End If '-- Options, Imports
Imports System.ComponentModel
Imports System.Runtime.CompilerServices
Imports System.Windows.Media
Imports System.Windows.Media.Imaging
Imports System.Windows.Controls
#End Region 'FileHeader

Namespace System

   Public Class MenuNode : Inherits Node(Of MenuNode)

      Public ReadOnly Property ImageWidth As Double
         Get
            Return DirectCast(Tree, MenuTree).ImageWidth
         End Get
      End Property

      Public Sub New()
      End Sub
      Public Sub New(name As String, execute As Action(Of Object), Optional canExecute As Func(Of Object, Boolean) = Nothing)
         Me.Name = name
         Command = New RelayCommand(Of Object)(execute, canExecute)
      End Sub

      Public Event Execute As EventHandler(Of ExecuteEventargs)
      Public Event CanExecute As EventHandler(Of CanExecuteEventargs)
      Public ID As String

      Public Function _CanExecute(itm As Object) As Boolean
         Dim e = New CanExecuteEventargs With {.Cancel = Not Command.Enabled, .ID = ID, .Param = itm}
         RaiseEvent CanExecute(Me, e)
         Return Not e.Cancel
      End Function
      Public Sub _Execute(itm As Object)
         Dim e = New ExecuteEventargs With {.ID = ID, .Param = itm}
         RaiseEvent Execute(Me, e)
      End Sub

      Public Property Visibility As Visibility
      Public Property ImageSource As ImageSource

      Public Property Command As RelayCommand(Of Object)

      Public ReadOnly Property Image As Image
         Get
            If ImageSource.Null Then Return Nothing
            Return New Image With {.Source = ImageSource, .Width = ImageWidth}
         End Get
      End Property
   End Class

   Public Class ExecuteEventargs : Inherits EventArgs
      Public ID As String
      Public Param As Object
   End Class

   Public Class CanExecuteEventargs : Inherits ExecuteEventargs
      Public Cancel As Boolean
   End Class

   Public Class MenuTree : Inherits Tree(Of MenuNode)

      Public ImageWidth As Double = 16

      Private Sub Add(name As String, ID As String)
         Dim nd = New MenuNode With {.Name = name, .ID = ID}
         Dim cmd = New RelayCommand(Of Object)(AddressOf nd._Execute, AddressOf nd._CanExecute)
         nd.Command = cmd
         Nodes.Add(nd)
         AddHandler nd.CanExecute, AddressOf Menu_CanExecute
         AddHandler nd.Execute, AddressOf Menu_Execute
      End Sub

      Private Shared _UriRoot As String = Nothing

      Private Function UriRoot() As String
         If _UriRoot.Null Then
            Dim thisAssembly = Reflection.Assembly.GetAssembly(Me.GetType).FullName.LeftCut(",")
            _UriRoot = "pack://application:,,,/".And(thisAssembly, ";component/Resources/")
         End If
         Return _UriRoot
      End Function

      Public Event Execute As EventHandler(Of ExecuteEventargs)
      Public Event CanExecute As EventHandler(Of CanExecuteEventargs)

      Public Sub New()
         Dim tp = Me.GetType
         Dim orders = tp.GetFields(Reflection.BindingFlags.Static Or Reflection.BindingFlags.Public)
         For Each order In orders
            Dim s = DirectCast(order.GetValue(Nothing), String)
            If s Is Nothing Then s = order.Name
            Add(s, order.Name)
            order.SetValue(Nothing, order.Name)
         Next
      End Sub

      Public Sub ExecuteById(id As String, Optional itm As Object = Nothing)
         Dim mnu = MyBase.ToList.FirstOrDefault(Function(nd) nd.Name = id)
         mnu._Execute(itm)
      End Sub

      Default Public ReadOnly Property SetImage(Id As String, file As String) As MenuTree
         Get
            Dim nd = ToList.First(Function(n) n.ID = Id)
            Try
               nd.ImageSource = New BitmapImage(New Uri(UriRoot() & file))
            Catch ex As Exception
               Dbg(Me.GetType.Name.And("-Exception: ", ex.Message))
            End Try
            Return Me
         End Get
      End Property


      Protected Sub SubMenu(parentID As String, ParamArray childIDs As String())
         Dim lst = ToList()
         Dim find As Func(Of String, MenuNode) = Function(id) lst.First(Function(n) n.ID = id)
         Dim ndparend = find(parentID)
         childIDs.ForEach(Sub(ID) ndparend.Add(find(ID)))
      End Sub

      Private Sub Menu_CanExecute(sender As Object, e As CanExecuteEventargs)
         RaiseEvent CanExecute(sender, e)
      End Sub

      Private Sub Menu_Execute(sender As Object, e As ExecuteEventargs)
         RaiseEvent Execute(sender, e)
      End Sub


   End Class
End Namespace