﻿#Region "FileHeader"
#If False Then
3 Datatypes in this File:
1) GridlengthEx combines a GridLength with Min and Max
2) GridLengthExsConverter converts a string to GridlengthEx[]
3) GridEx: provides a simplified Xaml-Syntax to define Grid-Columns and -Rows, and to place controls in it
3a) RowDefsProperty, ColumnDefsProperty As GridlengthEx[]: <Xaml>ColumnDefs = ",,auto min30,20,2*,max300"</Xaml> to define 7 differentiated ColumnDefinitions
3b) Attached RangeProperty: <Xaml>hlp:GridEx.Range = "a1 b2"</Xaml> places the Control in cells(0,0) with column-/row-span(2,2) Its Excel-like: 1/a - based counting
#End If
'-- Options, Imports
Imports System.ComponentModel
Imports tpSelf = System.Windows.Controls.GridEx
Imports System.Globalization
#End Region 'FileHeader

Namespace System.Windows.Controls

   Public Structure GridLengthEx
      Public Length As GridLength
      Public Min, Max As Double
   End Structure

   Public Class GridLengthExsConverter : Inherits TypeConverter

      Protected Shared ReadOnly _tpString As Type = GetType(String)
      Protected Shared _SplitChars As Char() = {","c}
      Protected Shared _Conv As TypeConverter = TypeDescriptor.GetConverter(GetType(GridLength))

      Public Overrides Function CanConvertFrom(context As ITypeDescriptorContext, sourceType As Type) As Boolean
         Return sourceType Is _tpString
      End Function

      ''' <summary> parst solche Strings: ",*,auto min30,20,2*,max300" in ein GridlengthEx[] </summary>
      Public Overrides Function ConvertFrom(context As ITypeDescriptorContext, culture As CultureInfo, value As Object) As Object
         Dim splits = DirectCast(value, String).Split(_SplitChars, StringSplitOptions.None)
         Return splits.ToArray(AddressOf ItemFromString)
      End Function
      Private Function ItemFromString(str As String) As GridLengthEx
         ItemFromString.Max = Double.PositiveInfinity
         If str = "" Then str = "*"
         For Each s In str.Split
            If s.StartsWith("min", StringComparison.OrdinalIgnoreCase) Then
               ItemFromString.Min = Double.Parse(s.Substring(3))
            ElseIf s.StartsWith("max", StringComparison.OrdinalIgnoreCase) Then
               ItemFromString.Max = Double.Parse(s.Substring(3))
            Else
               ItemFromString.Length.Be(_Conv.ConvertFrom(s))
            End If
         Next
      End Function
   End Class

   Public Class GridEx : Inherits Grid

      Public Shared ReadOnly RowDefsProperty As DependencyProperty = New DepProp(AddressOf RowDefsChanged)
      <Browsable(False), TypeConverter(GetType(GridLengthExsConverter))> _
      Public Property RowDefs() As GridLengthEx()
         Get
            Return DirectCast(GetValue(RowDefsProperty), GridLengthEx())
         End Get
         Set(value As GridLengthEx())
            SetValue(RowDefsProperty, value)
         End Set
      End Property
      Private Shared Sub RowDefsChanged(obj As DependencyObject, ev As DependencyPropertyChangedEventArgs)
         DirectCast(obj, tpSelf).RowDefsChanged(DirectCast(ev.NewValue, GridLengthEx()))
      End Sub
      Private Sub RowDefsChanged(value As GridLengthEx())
         Dim rowdefs = Array.ConvertAll(value, Function(glx) New RowDefinition() With {.Height = glx.Length, .MinHeight = glx.Min, .MaxHeight = glx.Max})
         RowDefinitions.ReFillBy(rowdefs)
      End Sub
      'Private Sub RowDefsChanged(value As GridLengthEx())
      '   RowDefinitions.ReFillBy(value.ToArray(Function(glx) New RowDefinition() With {.Height = glx.Length, .MinHeight = glx.Min, .MaxHeight = glx.Max}))
      'End Sub

      Public Shared ReadOnly ColumnDefsProperty As DependencyProperty = New DepProp(AddressOf ColumnDefsChanged)
      <Browsable(False), TypeConverter(GetType(GridLengthExsConverter))> _
      Public Property ColumnDefs() As GridLengthEx()
         Get
            Return DirectCast(GetValue(ColumnDefsProperty), GridLengthEx())
         End Get
         Set(value As GridLengthEx())
            SetValue(ColumnDefsProperty, value)
         End Set
      End Property
      Private Shared Sub ColumnDefsChanged(obj As DependencyObject, ev As DependencyPropertyChangedEventArgs)
         DirectCast(obj, tpSelf).ColumnDefsChanged(DirectCast(ev.NewValue, GridLengthEx()))
      End Sub
      Private Sub ColumnDefsChanged(value As GridLengthEx())
         Dim coldefs = Array.ConvertAll(value, Function(glx) New ColumnDefinition() With {.Width = glx.Length, .MinWidth = glx.Min, .MaxWidth = glx.Max})
         ColumnDefinitions.ReFillBy(coldefs)
      End Sub
      'Private Sub ColumnDefsChanged(value As GridLengthEx())
      '   ColumnDefinitions.ReFillBy(value.ToArray(Function(glx) New ColumnDefinition() With {.Width = glx.Length, .MinWidth = glx.Min, .MaxWidth = glx.Max}))
      'End Sub

      Public Shared ReadOnly RangeProperty As DependencyProperty = New DepProp("a1", AddressOf RangeCallback)
      <Description("sample: ""a1 b2"" would define cells(0,0) with column-/row-span(2,2)")> _
      <AttachedPropertyBrowsableForChildren()> _
      <Category("Layout")> _
      Public Shared Function GetRange(target As UIElement) As String
         Return DirectCast(target.GetValue(RangeProperty), String)
      End Function
      <AttachedPropertyBrowsableForChildren()> _
      <Category("Layout")> _
      Public Shared Sub SetRange(target As UIElement, value As String)
         target.SetValue(RangeProperty, value)
      End Sub

      Private Shared Sub RangeCallback(sender As DependencyObject, e As DependencyPropertyChangedEventArgs)
         Dim s = DirectCast(If(e.NewValue, e.Property.GetMetadata(GetType(GridEx)).DefaultValue), String)
         Dim splits = s.ToLower.Split({" "c}, StringSplitOptions.RemoveEmptyEntries)
         Dim xy = Parse(splits(0))
         Dim wh = If(splits.Length > 1, Parse(splits(1)), {-1, -1})
         For i = 0 To 1
            If xy(i) < 0 Then
               xy(i) = 0
               If wh(i) < 0 Then wh(i) = Short.MaxValue
            Else
               If wh(i) < 0 Then wh(i) = 0
            End If
            wh(i) += 1
         Next
         sender.SetValue(ColumnProperty, xy(0))
         sender.SetValue(RowProperty, xy(1))
         sender.SetValue(ColumnSpanProperty, wh(0))
         sender.SetValue(RowSpanProperty, wh(1))
      End Sub
      Private Shared Function Parse(s As String) As Integer()
         Dim x = Microsoft.VisualBasic.AscW(s) - 97
         Dim y = 0
         Integer.TryParse(s.Substring(If(x < 0, 0, 1)), y)
         Return {x, y - 1}
      End Function

   End Class
End Namespace

